# -*- coding: utf-8 -*-
"""
/***************************************************************************
Sample Layer Plugin
A QGIS plugin
Plugin using plugin layers

                             -------------------
begin                : 2009-11-30
copyright            : (C) 2009 by Mathias Walker, Sourcepole
email                : mwa at sourcepole.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *

import resources

LAYER_TYPE = "sample"

class SamplePluginLayer(QgsPluginLayer):
  def __init__(self, width=None):
    QgsPluginLayer.__init__(self, LAYER_TYPE, "Sample plugin layer")

    self.width = width if width is not None else 256
    self.setValid(True)

  def draw(self, rendererContext):
    painter = rendererContext.painter()
    painter.setPen(Qt.red)
    painter.drawRect(32, 32, self.width, 128)
    return True

  def readXml(self, node):
    self.width = node.toElement().attribute("width", "256").toInt()[0]
    return True

  def writeXml(self, node, doc):
    element = node.toElement()
    # write plugin layer type to project (essential to be read from project)
    element.setAttribute("type", "plugin")
    element.setAttribute("name", LAYER_TYPE)
    # custom properties
    element.setAttribute("width", str(self.width))
    return True

#------------------------------------------------------------------------------

class SamplePluginLayerType(QgsPluginLayerType):
  def __init__(self):
    QgsPluginLayerType.__init__(self, LAYER_TYPE)

  def createLayer(self):
    return SamplePluginLayer()

  def showLayerProperties(self, layer):
    res = QInputDialog.getInt(None, "Sample plugin", "Set width of the rectangle", layer.width, 1, 1000)
    if res[1]: # dialog was not cancelled
      layer.width = res[0]
      # trigger repaint
      layer.setCacheImage(None)
      layer.emit(SIGNAL("repaintRequested()"))

    # indicate that we have shown the properties dialog
    return True

#------------------------------------------------------------------------------

class SampleLayerPlugin:

  def __init__(self, iface):
    # Save reference to the QGIS interface
    self.iface = iface

  def initGui(self):
    # Create action that will start plugin configuration
    self.action = QAction(QIcon(":/plugins/sample_layer_plugin/icon.png"), "Add plugin layer", self.iface.mainWindow())
    # connect the action to the run method
    QObject.connect(self.action, SIGNAL("triggered()"), self.run)

    # Add toolbar button and menu item
    self.iface.addToolBarIcon(self.action)
    self.iface.addPluginToMenu("Sample layer plugin", self.action)

    QgsPluginLayerRegistry.instance().addPluginLayerType( SamplePluginLayerType() )

  def unload(self):
    # Remove the plugin menu item and icon
    self.iface.removePluginMenu("Sample layer plugin",self.action)
    self.iface.removeToolBarIcon(self.action)

    QgsPluginLayerRegistry.instance().removePluginLayerType(LAYER_TYPE)

  def run(self):
    layer = SamplePluginLayer()
    if layer.isValid():
      QgsMapLayerRegistry.instance().addMapLayer(layer)
